#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>

#include <jack/jack.h>

jack_port_t *output_port;

static int fs = 0;
static int done = 0;
static int step = 0;

float vals[] = {
	1.0f,		/* will be noise */
	0.0f,		/* zero, may cause denomrals following a signal */
	0.73 / 1e45,	/* very small - should be denormal when floated */
	3.7f,		/* arbitrary bigish number */
	-4.3f,		/* arbitrary bigish negative number */
	4294967395.0f,	/* 2^16 + 100 */
	-4294967395.0f,
	HUGE,		/* Big, non-inf number */
	1.0/0.0,	/* +inf */
	-1.0/0.0,	/* -inf */
	0.0		/* will be a NaN, sqrt(-1) */
};

static const int steps = sizeof(vals)/sizeof(float);

int
process (jack_nframes_t nframes, void *arg)

{
	static int count = 0;
	unsigned int pos;
	jack_default_audio_sample_t *out = (jack_default_audio_sample_t *) jack_port_get_buffer (output_port, nframes);

	if (step == 0) {
		for (pos = 0; pos < nframes; pos++) {
			out[pos] = (float)(rand() - rand())/(float)RAND_MAX;
		}
	} else {
		for (pos = 0; pos < nframes; pos++) {
			out[pos] = vals[step % steps];
		}
	}
	count += nframes;
	if (count > fs*3) {
		count = 0;
		step++;
	}

	if (step >= steps) {
		done = 1;
	}
		
	return 0;      
}

void
error (const char *desc)
{
	fprintf (stderr, "JACK error: %s\n", desc);
}

void
jack_shutdown (void *arg)
{
	exit (1);
}

int
main (int argc, char *argv[])
{
	jack_client_t *client;
	const char **ports;
	const char **port;
	int last;

	if (argc != 1) {
		fprintf (stderr, "usage: jack_demolition_client\n");
		return 1;
	}

	/* tell the JACK server to call error() whenever it
	   experiences an error.  Notice that this callback is
	   global to this process, not specific to each client.
	
	   This is set here so that it can catch errors in the
	   connection process
	*/
	jack_set_error_function (error);

	/* try to become a client of the JACK server */

	if ((client = jack_client_new ("demolition")) == 0) {
		fprintf (stderr, "jack server not running?\n");
		return 1;
	}

	/* tell the JACK server to call `process()' whenever
	   there is work to be done.
	*/

	jack_set_process_callback (client, process, 0);

	/* tell the JACK server to call `jack_shutdown()' if
	   it ever shuts down, either entirely, or if it
	   just decides to stop calling us.
	*/

	jack_on_shutdown (client, jack_shutdown, 0);

	/* display the current sample rate. once the client is activated 
	   (see below), you should rely on your own sample rate
	   callback (see above) for this value.
	*/

	fs = jack_get_sample_rate (client);

	/* create two ports */

	output_port = jack_port_register (client, "output", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);

	/* Add a NaN to the list of evil vals */

	vals[steps-1] = sqrt(-1.0);

	/* tell the JACK server that we are ready to roll */

	if (jack_activate (client)) {
		fprintf (stderr, "cannot activate client");
		return 1;
	}

	if ((ports = jack_get_ports (client, NULL, NULL, JackPortIsInput)) == NULL) {
		fprintf(stderr, "Cannot find any input ports\n");
		exit(1);
	}

	for (port = ports; *port; port++) {
		if (jack_connect (client, jack_port_name(output_port), *port)) {
			fprintf (stderr, "cannot connect to %s\n", *port);
		} else {
			printf("Connecting to %s\n", *port);
		}
	}

	free (ports);

	last = 0;
	printf("Outputing noise\n");
	while (!done) {
		if (step != last) {
			last = step;
			printf("Outputing %g\n", vals[step]);
		}
		usleep (1000);
	}
	jack_client_close (client);

	exit (0);
}
